@@ -15,12 +15,13 @@ import sysconfig
 import warnings
 from collections import OrderedDict
 from email.generator import BytesGenerator, Generator
+from email.policy import EmailPolicy
 from glob import iglob
 from io import BytesIO
 from shutil import rmtree
 from zipfile import ZIP_DEFLATED, ZIP_STORED
 
-import pkg_resources
+import setuptools
 from setuptools import Command
 
 from . import __version__ as wheel_version
@@ -28,13 +29,30 @@ from .macosx_libfile import calculate_macosx_platform_tag
 from .metadata import pkginfo_to_metadata
 from .util import log
 from .vendored.packaging import tags
+from .vendored.packaging import version as _packaging_version
 from .wheelfile import WheelFile
 
-safe_name = pkg_resources.safe_name
-safe_version = pkg_resources.safe_version
-setuptools_major_version = int(
-    pkg_resources.get_distribution("setuptools").version.split(".")[0]
-)
+
+def safe_name(name):
+    """Convert an arbitrary string to a standard distribution name
+    Any runs of non-alphanumeric/. characters are replaced with a single '-'.
+    """
+    return re.sub("[^A-Za-z0-9.]+", "-", name)
+
+
+def safe_version(version):
+    """
+    Convert an arbitrary string to a standard version string
+    """
+    try:
+        # normalize the version
+        return str(_packaging_version.Version(version))
+    except _packaging_version.InvalidVersion:
+        version = version.replace(" ", ".")
+        return re.sub("[^A-Za-z0-9.]+", "-", version)
+
+
+setuptools_major_version = int(setuptools.__version__.split(".")[0])
 
 PY_LIMITED_API_PATTERN = r"cp3\d"
 
@@ -65,7 +83,7 @@ def get_flag(var, fallback, expected=True, warn=True):
                 "Config variable '{}' is unset, Python ABI tag may "
                 "be incorrect".format(var),
                 RuntimeWarning,
-                2,
+                stacklevel=2,
             )
         return fallback
     return val == expected
@@ -113,13 +131,15 @@ def safer_version(version):
 
 
 def remove_readonly(func, path, excinfo):
-    print(str(excinfo[1]))
+    remove_readonly_exc(func, path, excinfo[1])
+
+
+def remove_readonly_exc(func, path, exc):
     os.chmod(path, stat.S_IWRITE)
     func(path)
 
 
 class bdist_wheel(Command):
-
     description = "create a wheel distribution"
 
     supported_compressions = OrderedDict(
@@ -212,13 +232,16 @@ class bdist_wheel(Command):
             bdist_base = self.get_finalized_command("bdist").bdist_base
             self.bdist_dir = os.path.join(bdist_base, "wheel")
 
+        egg_info = self.distribution.get_command_obj("egg_info")
+        egg_info.ensure_finalized()  # needed for correct `wheel_dist_name`
+
         self.data_dir = self.wheel_dist_name + ".data"
         self.plat_name_supplied = self.plat_name is not None
 
         try:
             self.compression = self.supported_compressions[self.compression]
         except KeyError:
-            raise ValueError(f"Unsupported compression: {self.compression}")
+            raise ValueError(f"Unsupported compression: {self.compression}") from None
 
         need_options = ("dist_dir", "plat_name", "skip_build")
 
@@ -284,7 +307,9 @@ class bdist_wheel(Command):
             ):
                 plat_name = "linux_i686"
 
-        plat_name = plat_name.lower().replace("-", "_").replace(".", "_")
+        plat_name = (
+            plat_name.lower().replace("-", "_").replace(".", "_").replace(" ", "_")
+        )
 
         if self.root_is_pure:
             if self.universal:
@@ -397,7 +422,10 @@ class bdist_wheel(Command):
         if not self.keep_temp:
             log.info(f"removing {self.bdist_dir}")
             if not self.dry_run:
-                rmtree(self.bdist_dir, onerror=remove_readonly)
+                if sys.version_info < (3, 12):
+                    rmtree(self.bdist_dir, onerror=remove_readonly)
+                else:
+                    rmtree(self.bdist_dir, onexc=remove_readonly_exc)
 
     def write_wheelfile(
         self, wheelfile_base, generator="bdist_wheel (" + wheel_version + ")"
@@ -454,6 +482,7 @@ class bdist_wheel(Command):
             warnings.warn(
                 'The "license_file" option is deprecated. Use "license_files" instead.',
                 DeprecationWarning,
+                stacklevel=2,
             )
             files.add(metadata["license_file"][1])
 
@@ -528,14 +557,19 @@ class bdist_wheel(Command):
 
             # delete dependency_links if it is only whitespace
             dependency_links_path = os.path.join(distinfo_path, "dependency_links.txt")
-            with open(dependency_links_path) as dependency_links_file:
+            with open(dependency_links_path, encoding="utf-8") as dependency_links_file:
                 dependency_links = dependency_links_file.read().strip()
             if not dependency_links:
                 adios(dependency_links_path)
 
         pkg_info_path = os.path.join(distinfo_path, "METADATA")
+        serialization_policy = EmailPolicy(
+            utf8=True,
+            mangle_from_=False,
+            max_line_length=0,
+        )
         with open(pkg_info_path, "w", encoding="utf-8") as out:
-            Generator(out, mangle_from_=False, maxheaderlen=0).flatten(pkg_info)
+            Generator(out, policy=serialization_policy).flatten(pkg_info)
 
         for license_path in self.license_paths:
             filename = os.path.basename(license_path)
